#include "mh.h"

void StandardMH::birth( double t, Pattern& D )
{
   int i = whran.discrete_uniform(0, D.getSize());
   Event *e = prior->newEvent();
   double area = prior->getWidth() * prior->getHeight();

   if( whran.uniform() < ( prior->cond_intens( D, e, i ) * area ) )
   {
      D.position_add( e, i );
      double w = whran.uniform();
      tvec.add( new Transition( e, i, t, 1, w ) );
   }
   else delete e;
}

void StandardMH::death( double t, Pattern& D )
{
   if( D.getSize() > 0 ) {
      int i = whran.discrete_uniform(0, D.getSize() - 1);
      Event *e = D.position_rmv(i);
      double area = prior->getWidth() * prior->getHeight();

      if( ( area * prior->cond_intens( D, e, i) ) * whran.uniform() <  1.0 ) {
         double w = whran.uniform();
         tvec.add( new Transition( e, i, t, 0, w ) ); 
      }
      else { D.position_add( e, i ); }
   }
}

void StandardMH::extend( Pattern& D, double& time )
{
   double t = 0.0;

   do {
      if ( t + 1.0e-7 > time ) break;

      double whr = whran.uniform();
      if( whr < 0.5 ) {
         birth( t, D );
      }
      else {
         death( t, D );
      }
      t += 1.0;
   } while (1);
}

void StandardMH::init( Pattern& D )
{
/* or empty if non-valid patterns are to be avoided  */

   double lambda = prior->getWidth() * prior->getHeight();
   lambda *= prior->ubnd();

   int n = whran.poisson( lambda );

   D.setSize(0);
   for( int i=0; i<n; i++) {
      Event *e = prior->newEvent();
      D.add(e);
   }
}

void StandardMH::sim( Pattern& D ) 
{
   double t = (double) time;
   extend( D, t );

   totalJumps = time;
   totalTime = t;
}

