#ifndef _noise_
#define _noise_ 1

/*
	Noise		parameters & rules for independent noise model
			- typically maintains noise variance parameters
			- yields likelihood ratio for signal
			  at each pixel
*/

#include <clip.short.h>
#include <clip.bin.h>
#include <clip.float.h>
#include <iostream>
#include <cmath>

class Noise {  
public:
   virtual double logLratio(int, double, double) const = 0; 
      // logLratio(y, theta1, theta0) is the log likelihood ratio
      // of parameter (= signal) value theta1 against theta0 
   virtual double logCanonic(int, double) const = 0;
      // for exponential models, the logarithm of the exponent
   virtual void report() const = 0;
};

class Gauss : public Noise {
   float var;        // variance 
public:
   Gauss( double cv ) { var = (float) cv; }

   double getVar() const { return var; }
   double setVar( double v) { var = (float) v; }

   double logLratio(int y, double t1, double t0) const;
   double logCanonic(int y, double t) const;

   void report() const { 
      std::cout << "Additive Gaussian noise with variance " << var << "\n";
   }
};

class Laplace : public Noise {
   float disp; 		// dispersion parameter
public:
   Laplace( double cd ) { disp = (float) disp; }

   double getDispersion() const { return disp; }
   double setDispersion( double d) { disp = (float) d; }

   double logLratio(int y, double t1, double t0) const;
   double logCanonic(int y, double t) const;

   void report() const { 
      std::cout << "Additive Laplacian noise with dispersion " << disp << "\n";
   }
};

#endif
