#include "pattern.h"
#include "random.h"

int nEvents;

double distance( const Event& a, const Event& b )
{
   double dx = b.getX() - a.getX();
   double dy = b.getY() - a.getY();
   return sqrt( dx*dx + dy*dy );
}

std::ostream& operator <<( std::ostream& s, Event p )
{
   return s << p.getX() << " " << p.getY();
}


Pattern::Pattern( int init_capacity ) : Vector( init_capacity )
{
  ;
}

void Pattern::position_add( EventPointer e, int k )
{
   int n = getSize();
   if( (k<0) || (k>n) ) {
      std::cout<<"index in Pattern::position_add()!\n";
      exit(1);
   }

   if( k == n ) add(e);
   else {
      add(NULL);
      for( int i=n; i>k; i-- ) set(i, get(i-1));
      set(k, e);
   }
   setSize(n+1);
}

EventPointer Pattern::position_rmv( int k )
{
   int n = getSize();
   if( (k<0) || (k>=n) ) {
      std::cout<<"index in Pattern::position_rmv()!\n";
      exit(1);
   }

   Event *e = getEvent(k);
   for( int i=k; i<n-1; i++ ) set(i, get(i+1));
   setSize(n-1);

   return e;
}


void Pattern::destroy()
{
   int n = getSize();
   Event *e;
   for( int k=0; k<n; k++ ) {
      e = getEvent(k);
      delete e;

      nEvents --;
   }
   setSize(0);

}

void VectorInt::destroy()
{
   int n = getSize();
   IntVal *iv;
   for( int k=0; k<n; k++ ) {
      iv = (IntVal *) this->get(k);
      delete iv;
   }
   setSize(0); 

/*
   while (getSize()>0)
        {
       IntValPointer last=remove_last();
       delete last;
      }
*/
}

VectorInt::VectorInt( int init_capacity ) : Vector( init_capacity )
{
   ;
}
Transition::Transition( Event *e, int pos, double t, short is_birth, double w )
{
   this->e = e;
   this->pos = pos;
   this->t = t;
   this->is_birth = is_birth;
   this->w = w;
   prev = NULL;
}

PermuteTransition::PermuteTransition( Event *e, int pos, double t, 
  short is_birth, double w ) : Transition( e, pos, t, is_birth, w ), L(5)
{
   this->u = e;
   this->l = NULL;
}

void PermuteTransition::setPreviousEvents( 
   Event *eUp, Event *eLp, Pattern Lp)
{
   u = eUp;
   l = eLp;
   L = Lp;
}

PermuteTransition::~PermuteTransition()
{
//   cout << "destructor PermuteTransition\n";
//   cout << "nop_del = " <<  nop_del << "\n";
}

void TransitionList::destroy()
{
//cout << "enter destroy\n";
   Transition *tp = last;
   while ( tp != NULL ) {
      last = last->prev;
      if( tp->is_birth == 1 ) { delete tp->e; nEvents --; }

//cout << "nop_del = " <<  nop_del << "\n";
      delete tp;
//cout << "nop_del = " <<  nop_del << "\n";
      tp = last;
   }
   size = 0;
//cout << "exit destroy\n";
}

/*
void TransitionList::destroy()
{
   Transition *tp = last;
   while ( tp != NULL ) {
      last = last->prev;
      if( tp->is_birth == 1 ) delete tp->e;

      if((tp->is_birth==0)&&(T.find(tp->e)!=-1))
        {
          T.remove(tp->e);
          delete tp->e;
        }
      
      delete tp;
      tp = last;
   }
   size=0;
}
*/
