#include "sampler.h"

Sampler::Sampler( Prior *prior, char *name_file )
{
   this->prior = prior;
   totalTime = 0.0;
   totalJumps = 0;

   if( (list_file=fopen(name_file,"a"))==NULL ) {
      std::cout << "ERROR : opening file " << name_file <<" \n";
      exit(1);
   }
}

Sampler::~Sampler()
{
   tvec.destroy();
   fclose(list_file);
}

void Sampler::birth( double t, Pattern& D )
{
   int i = whrg.discrete_uniform(0, D.getSize());
   Event *e = prior->newEvent();
   D.position_add( e, i );
   double w = whrg.uniform();
   tvec.add( new Transition( e, i, t, 1, w ) );
}

void Sampler::death( double t, Pattern& D )
{ 
   if( D.getSize() > 0 ) {
      int i = whrg.discrete_uniform(0, D.getSize() - 1);
      Event *e = D.position_rmv(i);
      double w = whrg.uniform();
      tvec.add( new Transition( e, i, t, 0, w ) );
   }
}

void Sampler::clean()
{
   tvec.destroy(); 
}

void Sampler::printTimeJumps()
{
   fprintf(list_file, "%f %d \n", getTime(), getJumps());
   fflush(list_file); 
}
