#include "event.h"

sequence square( int side )
{
   point top_left = point(0, 0);
   point low_right = point(side, side);
   return( seq( range(top_left, low_right) ) );
}

Square::Square( VideoObject *cp, VideoObject *cc, double xc, double yc, double sc, int ccol ) : 
   VideoObject( cp, cc, xc, yc, ccol ) 
{
   side = (float) sc;
   pixels = square( (int) round(side) );
}

Square::Square( double xc, double yc, double sc, int ccol ) : 
   VideoObject( xc, yc, ccol ) 
{
   side = (float) sc;
   pixels = square( (int) round(side) );
}

void Square::setSide( double sc ) 
{
   side = (float) sc;
   pixels = square( (int) round(side) );
}

int Square::overlap( const Object *e ) const
{
   const Square *f = (const Square *) e; 
   if( getX() + getSide() < f->getX() ) return 0;
   if( f->getX() + f->getSide() < getX() ) return 0;
   if( getY() + getSide() < f->getY() ) return 0;
   if( f->getY() + f->getSide() < getY() ) return 0;

   return 1;
}

int Square::contains( point p ) const
{
   int dside = (int) round( getSide() );
   int dx = (int) round( getX() );
   int dy = (int) round( getY() );

   if( ( dx + dside < p.x ) || ( p.x < dx ) ) return 0;
   if( ( dy + dside < p.y ) || ( p.y < dy ) ) return 0;

   return 1;
}


sequence Square::shape() const
{
   point p( (int) round(getX()), (int) round(getY()) );
   return ( pixels + p );
}


void Square::report() const
{
   std::cout << "Top left point of square: ( " 
             << getX() << ", " << getY()  << " )\n";
   std::cout << "Side: " << getSide() << "\n";
   std::cout << "Colour: " << getColour() << "\n";

   if( parent != NULL ) 
      std::cout << "Parent: ( " << parent->getX() << ", " 
                << parent->getY() << " )\n";
   if( child != NULL ) 
      std::cout << "Child: ( " << child->getX() << ", " 
                 << child->getY() << " )\n";
}

