#include "event.h"

Square::Square( VideoObject *cp, VideoObject *cc, double xc, double yc, double sc, int ccol ) : 
   VideoObject( cp, cc, xc, yc, ccol ) 
{
   side = (float) sc;
   int dside = (int) round(side);
   point top_left = point(0, 0);
   point low_right = point(dside-1, dside-1);

   pixels = seq( range(top_left, low_right) );
}

Square::Square( double xc, double yc, double sc, int ccol ) : 
   VideoObject( xc, yc, ccol ) 
{
   side = (float) sc;
   int dside = (int) round(side);
   point top_left = point(0, 0);
   point low_right = point(dside-1, dside-1);

   pixels = seq( range(top_left, low_right) );
}

void Square::setSide( double sc ) 
{
   side = (float) sc;
   int dside = (int) round(side);
   point top_left = point(0, 0);
   point low_right = point(dside-1, dside-1);

   pixels = seq( range(top_left, low_right) );
}


int Square::overlap( const Object *e ) const
{
   const Square *f = (const Square *) e; 
   int x = (int) round(getX() - getSide()/2.0);
   int fx = (int) round(f->getX() - f->getSide()/2.0);
   int y = (int) round(getY() - getSide()/2.0);
   int fy = (int) round(f->getY() - f->getSide()/2.0);
   if( x + getSide() - 1 < fx ) return 0;
   if( fx + f->getSide() - 1 < x ) return 0;
   if( y + getSide() - 1 < fy ) return 0;
   if( fy + f->getSide() - 1 < y ) return 0;

   return 1;
}

int Square::contains( point p ) const
{
   int x = (int) round(getX() - getSide()/2.0);
   int y = (int) round(getY() - getSide()/2.0);

   if( x + getSide() - 1 < p.x - 0.5 ) return 0;
   if( 0.5 + p.x < x ) return 0;
   if( y + getSide() - 1 < p.y - 0.5 ) return 0;
   if( 0.5 + p.y < y ) return 0;

   return 1;
}


sequence Square::shape() const
{
   point p( (int) round(getX() - getSide()/2.0), (int) round(getY() - getSide()/2.0) );
   return ( pixels + p );
}


void Square::report() const
{
   std::cout << "Centre of square: ( " << getX() << ", " << getY()  << " )\n";
   std::cout << "Side: " << getSide() << "\n";
   std::cout << "Colour: " << getColour() << "\n";
   if( parent != NULL ) std::cout << "Parent: ( " << parent->getX() << ", " << parent->getY() << " )\n";
   if( child != NULL ) std::cout << "Child: ( " << child->getX() << ", " << child->getY() << " )\n";
}

