#include "state.h"

void 
State::add( VideoObject *ob, int frame, int i ) // no links
{
   if( frame > nframe - 1 || frame < 0 ) { 
       std::cout << "State add\n"; exit(1); }
   if( i < 0 || i > (fs[frame].pat).getSize() ) {
       std::cout << "State add\n"; exit(1); }

   (fs[frame].sil)->add( fs[frame].pat, ob, i );
   (fs[frame].pat).position_add(ob, i);
}

void 
State::add_link( VideoObject *o1, VideoObject *o2 )
{
   if( o1->child == NULL && o2->parent == NULL ) { 
      o1->child = o2; 
      o2->parent = o1;
   }
   else { std::cout << "Child or parent link exists\n"; exit(1); }
}

void 
State::remove( int frame, int i ) // leaves links intact
{
   if( frame > nframe - 1 || frame < 0 ) { 
       std::cout << "State remove\n"; exit(1); }
   if( i < 0 || i > (fs[frame].pat).getSize() - 1 ) {
       std::cout << "State remove\n"; exit(1); }
   
   (fs[frame].sil)->del( fs[frame].pat, i );
   (fs[frame].pat).position_rmv(i);
}

void 
State::remove_link( VideoObject *o1, VideoObject *o2 )
{
   if( o1->child == o2 ) o1->child = NULL;
   else { std::cout << "Child link does not exists\n"; exit(1); }
   if( o2->parent == o1 ) o2->parent = NULL;
   else { std::cout << "Parent link does not exists\n"; exit(1); }
}


/* 
  contributions to Hastings ratio by prior and likelihood
  ON THE LOG SCALE
*/

double 
State::llr_birth( VideoObject *ob, int frame, int i ) 
{
   double llr = 0.0; 

   this->add( ob, frame, i );
   FOR( p in ob->shape() ) {
     if( has(p) ) llr += 
     noise->logCanonic( (fs[frame].data)(p), (fs[frame].sil)->val(p) );
   }

   this->remove( frame, i );
   FOR( p in ob->shape() ) {
      if( has(p) ) llr -= 
      noise->logCanonic( (fs[frame].data)(p), (fs[frame].sil)->val(p) );
   }
   
   return(llr);
}

double 
State::llr_death( int frame, int i ) 
{
   VideoObject *ob = (VideoObject *) (fs[frame].pat).getEvent(i);
   double llr = 0.0; 

   this->remove( frame, i );
   FOR( p in ob->shape() ) {
     if( has(p) ) llr += 
     noise->logCanonic( (fs[frame].data)(p), (fs[frame].sil)->val(p) );
   }

   this->add( ob, frame, i );
   FOR( p in ob->shape() ) {
      if( has(p) ) llr -= 
      noise->logCanonic( (fs[frame].data)(p), (fs[frame].sil)->val(p) );
   }

   return(llr);
}

double 
State::down_birth( const VideoObject *ob, int frame, int i, 
                   const VideoObject *prev ) 
{
   double llr = ( prior->logCond_intens( fs[frame].pat, ob, i ) + 
                  log( 1.0 + (fs[frame].pat).getSize() ) );

   llr += ( prior->logRho( prev, ob ) - prior->logLambda( prev ) );
   if( frame < nframe - 1 ) llr += prior->logLambda(ob);

   return( llr );
}

double 
State::up_birth( const VideoObject *ob, int frame, int i,
                 const VideoObject *next ) 
{
   double llr = ( prior->logCond_intens( fs[frame].pat, ob, i ) + 
                  log( 1.0 + (fs[frame].pat).getSize() ) );

   llr += ( prior->logRho(ob, next) - prior->logLambda( next ) );
   if( frame > 0 ) llr += prior->logLambda(ob);

   return( llr );
}

double 
State::double_birth( const VideoObject *ob, int frame, int i,
       const VideoObject *prev, const VideoObject *next )
{
   double llr = ( prior->logCond_intens( fs[frame].pat, ob, i ) + 
                  log( 1.0 + (fs[frame].pat).getSize() ) );

   llr += ( prior->logRho( ob, next ) + prior->logRho( prev, ob ) );
   llr -= ( prior->logLambda(next) + prior->logLambda(prev) );

   return( llr );
}

double 
State::unmatch_birth( const VideoObject *ob, int frame, int i ) 
{
   double llr = ( prior->logCond_intens( fs[frame].pat, ob, i ) + 
                  log( 1.0 + (fs[frame].pat).getSize() ) ); 

   if( ( frame == 0 ) || ( frame == nframe - 1 ) ) 
      return( llr + prior->logLambda(ob) );
   else return( llr + ( 2.0 * prior->logLambda(ob) ) );
}

double 
State::down_death( int frame, int i ) 
{
   VideoObject *ob = (VideoObject *) (fs[frame].pat).getEvent(i);
   this->remove( frame, i );

   double llr = - ( prior->logCond_intens( fs[frame].pat, ob, i) + 
                    log( 1.0 + (fs[frame].pat).getSize() ) );
   this->add( ob, frame, i );

   llr += ( prior->logLambda( ob->parent ) - prior->logRho( ob->parent, ob ) );
   if( frame < nframe - 1 ) llr -= prior->logLambda(ob);

   return( llr );
}


double 
State::up_death( int frame, int i ) 
{
   VideoObject *ob = (VideoObject *) (fs[frame].pat).getEvent(i);
   this->remove( frame, i );

   double llr = - ( prior->logCond_intens( fs[frame].pat, ob, i) + 
                    log( 1.0 + (fs[frame].pat).getSize() ) );
   this->add( ob, frame, i );

   llr += ( prior->logLambda( ob->child ) - prior->logRho( ob, ob->child ) );
   if( frame > 0 ) llr -= prior->logLambda(ob);

   return( llr );
}

double 
State::double_death( int frame, int i ) 
{
   VideoObject *ob = (VideoObject *) (fs[frame].pat).getEvent(i);
   this->remove( frame, i );

   double llr = - ( prior->logCond_intens( fs[frame].pat, ob, i) + 
                    log( 1.0 + (fs[frame].pat).getSize() ) );
   this->add( ob, frame, i );

   llr += ( prior->logLambda( ob->child ) + prior->logLambda( ob->parent ) );
   llr -= ( prior->logRho( ob->parent, ob ) + prior->logRho(ob, ob->child) );

   return( llr );
}

double 
State::unmatch_death( int frame, int i ) 
{
   VideoObject *ob = (VideoObject *) (fs[frame].pat).getEvent(i);
   this->remove( frame, i );

   double llr = - ( prior->logCond_intens( fs[frame].pat, ob, i) + 
                    log( 1.0 + (fs[frame].pat).getSize() ) );
   this->add( ob, frame, i );

   if( ( frame == 0 ) || ( frame == nframe - 1 ) )
      return( llr - prior->logLambda(ob) );
   else return( llr - ( 2.0 * prior->logLambda(ob) ) );
}


double 
State::birth_link( const VideoObject *ob, const VideoObject *prev ) const
{
   double llr = prior->logRho( prev, ob );
   llr -= ( prior->logLambda(ob) +  prior->logLambda(prev) ); 
   return( llr );
}

double 
State::death_link( const VideoObject *ob, const VideoObject *prev ) const
{
   double llr = prior->logLambda(ob) + prior->logLambda(prev);
   llr -= prior->logRho( prev, ob );
   return( llr );
}

double 
State::change( VideoObject *ob, VideoObject *tob, int i, int frame ) 
{
   double llr = llr_death( frame, i );

   this->remove( frame, i );
   llr -= ( prior->logCond_intens( fs[frame].pat, ob, i) + 
            log( 1.0 + (fs[frame].pat).getSize() ) );

   llr += llr_birth( tob, frame, i );

   llr += ( prior->logCond_intens( fs[frame].pat, tob, i) + 
            log( 1.0 + (fs[frame].pat).getSize() ) );
   this->add( tob, frame, i );

   if( frame > 0 ) {
      if( ob->parent == NULL ) {
         llr += ( prior->logLambda(tob) - prior->logLambda(ob) );
      }
      else {
        llr += ( prior->logRho(tob->parent, tob) - prior->logRho(ob->parent, ob) );
      }
   }

   if( frame < nframe - 1 ) {
      if( ob->child == NULL ) {
         llr += ( prior->logLambda(tob) - prior->logLambda(ob) );
      }
      else {
         llr += ( prior->logRho(tob, tob->child) - prior->logRho(ob, ob->child) );
      }
   }

   this->remove( frame, i);
   this->add( ob, frame, i );

   return( llr );
}

double 
State::split( VideoObject *ob, VideoObject *s1, VideoObject *s2, int link,
              int i, int frame ) 
{
   double llr = llr_death( frame, i );
   this->remove( frame, i );
   llr -= 
    ( prior->logCond_intens( (fs[frame].pat), ob, i ) + log( 1.0 + (fs[frame].pat).getSize() ) );

   llr += llr_birth( s1, frame, i );
   llr += 
    ( prior->logCond_intens( (fs[frame].pat), s1, i ) + log( 1.0 + (fs[frame].pat).getSize() ) );
   this->add( s1, frame, i ); 

   llr += llr_birth( s2, frame, i+1 );
   llr += 
    ( prior->logCond_intens( (fs[frame].pat), s2, i+1 ) + log( 1.0 + (fs[frame].pat).getSize() ) );
   this->add( s2, frame, i+1 ); 

   
   if( link == 1 ) {
      if( frame > 0 ) {
         if( ob->parent == NULL ) llr +=
           ( prior->logLambda(s1) - prior->logLambda(ob) );
         else llr += ( prior->logRho(ob->parent, s1) -
                       prior->logRho(ob->parent, ob) );
      }
      if( frame < nframe - 1 ) {
         if( ob->child == NULL ) llr +=
           ( prior->logLambda(s1) - prior->logLambda(ob) );
         else llr += ( prior->logRho(s1, ob->child) -
                       prior->logRho(ob, ob->child) );
      }
      if( s2->child != NULL ) llr +=
        ( prior->logRho(s2, s2->child) - prior->logLambda(s2->child) );
      else { if( frame < nframe -1 ) llr += prior->logLambda(s2); }
      if( s2->parent != NULL ) llr +=
        ( prior->logRho(s2->parent, s2) - prior->logLambda(s2->parent) );
      else { if( frame > 0 ) llr += prior->logLambda(s2); }
   }

   if( link == 2 ) {
      if( frame > 0 ) {
         if( ob->parent == NULL ) llr +=
           ( prior->logLambda(s2) - prior->logLambda(ob) );
         else llr += ( prior->logRho(ob->parent, s2) -
                       prior->logRho(ob->parent, ob) );
      }
      if( frame < nframe - 1 ) {
         if( ob->child == NULL ) llr +=
           ( prior->logLambda(s2) - prior->logLambda(ob) );
         else llr += ( prior->logRho(s2, ob->child) -
                       prior->logRho(ob, ob->child) );
      }
      if( s2->child != NULL ) llr +=
        ( prior->logRho(s2, s2->child) - prior->logLambda(s2->child) );
      else { if( frame < nframe -1 ) llr += prior->logLambda(s2); }
      if( s2->parent != NULL ) llr +=
        ( prior->logRho(s2->parent, s2) - prior->logLambda(s2->parent) );       else { if( frame > 0 ) llr += prior->logLambda(s2); }
   }

   this->remove( frame, i+1 );
   this->remove( frame, i );
   this->add( ob, frame, i );

   return( llr );
}

double 
State::merge( VideoObject *ob1, VideoObject *ob2, VideoObject *mob, int link, 
              int i, int frame ) 
{
   double llr = llr_death( frame, i );
   this->remove( frame, i );
   llr -= 
    ( prior->logCond_intens( fs[frame].pat, ob1, i ) + log( 1.0 + (fs[frame].pat).getSize() ) );

   llr += llr_death( frame, i );
   this->remove( frame, i );
   llr -= 
    ( prior->logCond_intens( fs[frame].pat, ob2, i ) + log( 1.0 + (fs[frame].pat).getSize() ) );

   llr += llr_birth( mob, frame, i );
   llr += 
    ( prior->logCond_intens( fs[frame].pat, mob, i ) + log( 1.0 + (fs[frame].pat).getSize() ) );
   this->add( mob, frame, i ); 

   if( link == 1 ) {
      if( frame > 0 ) {
         if( mob->parent == NULL ) llr += 
           ( prior->logLambda(mob) - prior->logLambda(ob1) );
         else llr += ( prior->logRho(ob1->parent, mob) - 
                       prior->logRho(ob1->parent, ob1) );
      }
      if( frame < nframe - 1 ) {
         if( mob->child == NULL ) llr += 
           ( prior->logLambda(mob) - prior->logLambda(ob1) );
         else llr += ( prior->logRho(mob, ob1->child) -
                       prior->logRho(ob1, ob1->child) );
      }
      if( ob2->child != NULL ) llr += 
        ( prior->logLambda(ob2) - prior->logRho(ob2, ob2->child) ); 
      else { if( frame < nframe -1 ) llr -= prior->logLambda(ob2); }
      if( ob2->parent != NULL ) llr += 
        ( prior->logLambda(ob2) - prior->logRho(ob2->parent, ob2) );
      else { if( frame > 0 ) llr -= prior->logLambda(ob2); }
   }

   if( link == 2 ) {
      if( frame > 0 ) {
         if( mob->parent == NULL ) llr += 
           ( prior->logLambda(mob) - prior->logLambda(ob2) );
         else llr += ( prior->logRho(ob2->parent, mob) -
                       prior->logRho(ob2->parent, ob2) );
      }
      if( frame < nframe - 1 ) {
         if( mob->child == NULL ) llr += 
           ( prior->logLambda(mob) - prior->logLambda(ob2) );
         else llr += ( prior->logRho(mob, ob2->child) -
                       prior->logRho(ob2, ob2->child) );
      }
      if( ob1->child != NULL ) llr +=
        ( prior->logLambda(ob1) - prior->logRho(ob1, ob1->child) );
      else { if( frame < nframe -1 ) llr -= prior->logLambda(ob1); }
      if( ob1->parent != NULL ) llr +=
        ( prior->logLambda(ob1) - prior->logRho(ob1->parent, ob1) );
      else { if( frame > 0 ) llr -= prior->logLambda(ob1); }
   }

   this->remove( frame, i );
   this->add( ob2, frame, i );
   this->add( ob1, frame, i );

   return( llr );
}


/*
   MH iteration
*/

void 
State::birth_singly_up( int frame, int m_s, int m_d, double T )
{
   if( frame > nframe - 2 ) return;

   Pattern child = fs[frame + 1].pat;
   if( child.getSize() - m_s - m_d < 1 ) return;

   VideoObject *next;
   int pos_next = whrg.discrete_uniform( 1, child.getSize() - m_s - m_d );
   int m = 0;
   for( int j=0; j<child.getSize(); j++ ) {
      next = (VideoObject *) child.getEvent(j);
      if( next->parent == NULL ) m++;
      if( m == pos_next ) break;
   }

   int i = whrg.discrete_uniform(0, (fs[frame].pat).getSize());
   VideoObject *ob = (VideoObject*) prior->from_kernel( next );

   double energy =
      ( ( llr_birth( ob, frame, i ) + up_birth( ob, frame, i, next ) ) / T );

   double accept =
      energy + log( child.getSize() - m_s - m_d );

   accept -= ( log( 1.0 + m_s ) + prior->logKernel( ob, next ) );

   if( log(whrg.uniform()) < accept ) {
      add( ob, frame, i );
      add_link( ob, next );
      Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 )  iwait();
   }
   else delete ob;

   return;
}


void
State::birth_singly_down( int frame, int m_s, int m_d, double T )
{
   if( frame < 1 ) return; 

   Pattern parent = fs[frame - 1].pat;
   if( parent.getSize() - m_s - m_d < 1 ) return;

   VideoObject *prev;
   int pos_prev = whrg.discrete_uniform( 1, parent.getSize() - m_s - m_d );
   int m = 0;
   for( int j=0; j<parent.getSize(); j++ ) {
      prev = (VideoObject *) parent.getEvent(j);
      if( prev->child == NULL ) m++;
      if( m == pos_prev ) break;
   }

   int i = whrg.discrete_uniform(0, (fs[frame].pat).getSize());
   VideoObject *ob = (VideoObject*) prior->from_kernel( prev );

   double energy = 
      ( ( llr_birth( ob, frame, i ) + down_birth( ob, frame, i, prev ) ) / T );

   double accept = 
      energy + log( parent.getSize() - m_s - m_d );
   accept -= ( log( 1.0 + m_s ) + prior->logKernel( ob, prev ) ); 

   if( log(whrg.uniform()) < accept ) {
     add( ob, frame, i );
     add_link( prev, ob );
     Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
   }
   else delete ob; 

   return;
}

void
State::birth_doubly( int frame, int m_su, int m_sd, int m_d, double T )
{
   if( frame < 1 || frame > nframe - 2 ) return;

   Pattern child = fs[frame + 1].pat;
   if( child.getSize() - m_su - m_d < 1 ) return;

   Pattern parent = fs[frame - 1].pat;
   if( parent.getSize() - m_sd - m_d < 1 ) return;

   VideoObject *next;
   int pos_next = whrg.discrete_uniform( 1, child.getSize() - m_su - m_d );
   VideoObject *prev;
   int pos_prev = whrg.discrete_uniform( 1, parent.getSize() - m_sd - m_d );

   int j;
   int m = 0;
   for( j=0; j<child.getSize(); j++ ) {
      next = (VideoObject *) child.getEvent(j);
      if( next->parent == NULL ) m++;
      if( m == pos_next ) break;
   }

   m = 0;
   for( j=0; j<parent.getSize(); j++ ) {
      prev = (VideoObject *) parent.getEvent(j);
      if( prev->child == NULL ) m++;
      if( m == pos_prev ) break;
   }

   int i = whrg.discrete_uniform(0, (fs[frame].pat).getSize());
   VideoObject *ob = (VideoObject*) prior->from_kernel( prev );

   double energy = 
      ( ( llr_birth( ob, frame, i ) + double_birth( ob, frame, i, prev, next ) ) / T );

   double accept = energy + 
     ( log( parent.getSize() - m_sd - m_d ) +
       log( child.getSize() - m_su - m_d ) );
   accept -= ( log( 1.0 + m_d ) + prior->logKernel( ob, prev ) );

   if( log(whrg.uniform()) < accept ) {
      add( ob, frame, i );
      add_link( prev, ob );
      add_link( ob, next );
      Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
   }
   else delete ob; 

   return;
}

void 
State::birth_matched( double T )
{
   VideoObject *e;

   int m_singly_up = 0;  // nr in current frame connected to next frame only
   int m_singly_down = 0;  // those connected to previous frame only
   int m_doubly = 0;  // and to both next and previous frames

   int frame = whrg.discrete_uniform(0, nframe-1);
   Pattern current = fs[frame].pat;
   for( int j=0; j<current.getSize(); j++ ) {
      e = (VideoObject *) current.getEvent(j);
      if( e->child != NULL && e->parent == NULL ) m_singly_up++;
      if( e->child == NULL && e->parent != NULL ) m_singly_down++;
      if( e->child != NULL && e->parent != NULL ) m_doubly++;
   }

   if( (3.0*whrg.uniform()) < 2.0  ) { // singly 
      if( whrg.uniform() < 0.5 ) { // up
         birth_singly_up( frame, m_singly_up, m_doubly, T );
      }
      else { // down
         birth_singly_down( frame, m_singly_down, m_doubly, T );
      }
   }
   else { // doubly
      birth_doubly( frame, m_singly_up, m_singly_down, m_doubly, T);
   }

   return;
}

void
State::death_singly_up( int frame, int m_s, int m_d, double T )
{
   if( frame > nframe - 2 ) return;
   if( m_s < 1 ) return;

   Pattern current = fs[frame].pat;

   VideoObject *ob;
   VideoObject *next;
   int pos_next = whrg.discrete_uniform( 1, m_s );
   int m = 0;
   for( int j=0; j<current.getSize(); j++ ) {
      ob = (VideoObject *) current.getEvent(j);
      if( ob->child != NULL && ob->parent == NULL ) m++;
      if( m == pos_next ) { 
         pos_next = j; next = (VideoObject *) ob->child; 
         break; 
      }
   }

   double energy = 
      ( ( llr_death( frame, pos_next ) + up_death( frame, pos_next ) ) / T );

   double accept = 
      energy + log( m_s) + prior->logKernel( ob, next );
   accept -= log( (fs[frame+1].pat).getSize() - m_s - m_d + 1.0 );

   if( log(whrg.uniform()) < accept ) {
      remove_link( ob, next );
      remove( frame, pos_next );
      Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) { report_all(); iwait(); }
      delete ob;
   }

   return;
}

void
State::death_singly_down( int frame, int m_s, int m_d, double T )
{
   if( frame < 1 ) return;
   if( m_s < 1 ) return;

   Pattern current = fs[frame].pat;

   VideoObject *ob; 
   VideoObject *prev;
   int pos_prev = whrg.discrete_uniform( 1, m_s );
   int m = 0;
   for( int j=0; j<current.getSize(); j++ ) {
      ob = (VideoObject *) current.getEvent(j);
      if( ob->parent != NULL && ob->child == NULL ) m++;
      if( m == pos_prev ) {
         pos_prev = j; prev = (VideoObject *) ob->parent; 
         break;
      }
   }

   double energy = 
      ( ( llr_death( frame, pos_prev ) + down_death( frame, pos_prev ) ) / T );

   double accept = 
      energy + log( m_s ) + prior->logKernel( ob, prev );
   accept -= log( (fs[frame-1].pat).getSize() - m_s - m_d + 1.0 );

   if( log(whrg.uniform()) < accept ) {
      remove_link( prev, ob );
      remove( frame, pos_prev );
      Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
      delete ob;
   }

   return;
}

void
State::death_doubly( int frame, int m_su, int m_sd, int m_d, double T )
{
   if( frame < 1 || frame > nframe - 2 ) return;
   if( m_d < 1 ) return;

   Pattern current = fs[frame].pat;

   VideoObject *ob; 
   VideoObject *prev; VideoObject *next;
   int pos_doubly = whrg.discrete_uniform( 1, m_d );
   int m = 0;
   for( int j=0; j<current.getSize(); j++ ) {
      ob = (VideoObject *) current.getEvent(j);
      if( ob->child != NULL && ob->parent != NULL ) m++;
      if( m == pos_doubly ) {
         pos_doubly = j; prev = (VideoObject *) ob->parent; 
         next = (VideoObject *) ob->child; break;
      }
   }

   double energy = 
      ( ( llr_death( frame, pos_doubly ) + double_death( frame, pos_doubly ) ) / T );

   double accept = 
      energy + log( m_d ) + prior->logKernel( ob, prev );
   accept -= log( (fs[frame-1].pat).getSize() - m_sd - m_d + 1.0 );
   accept -= log( (fs[frame+1].pat).getSize() - m_su - m_d + 1.0 );

   if( log(whrg.uniform()) < accept ) {
      remove_link( prev, ob );
      remove_link( ob, next );
      remove( frame, pos_doubly );
      Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
      delete ob;
   }

   return;
}

void 
State::death_matched( double T )
{
   VideoObject *e;

   int m_singly_up = 0; 
   int m_singly_down = 0; 
   int m_doubly = 0;

   int frame = whrg.discrete_uniform(0, nframe-1);
   Pattern current = fs[frame].pat;
   for( int j=0; j<current.getSize(); j++ ) {
        e = (VideoObject *) current.getEvent(j);
        if( e->child != NULL && e->parent == NULL ) m_singly_up++;
        if( e->child == NULL && e->parent != NULL ) m_singly_down++;
        if( e->child != NULL && e->parent != NULL ) m_doubly++;
   }
   
   if( (3.0*whrg.uniform()) < 2.0  ) { // singly 
      if( whrg.uniform() < 0.5 ) { // up
         death_singly_up( frame, m_singly_up, m_doubly, T );
      }
      else { // down
         death_singly_down( frame, m_singly_down, m_doubly, T );
      }
   }
   else { // doubly
      death_doubly( frame, m_singly_up, m_singly_down, m_doubly, T );
   }

   return;
}

void 
State::birth_unmatched( double T )
{
   int frame = whrg.discrete_uniform(0, nframe-1);
   Pattern current = fs[frame].pat;
   VideoObject *ob = (VideoObject*) prior->newEvent();
   int i = whrg.discrete_uniform(0, current.getSize());

   double energy = 
       ( llr_birth( ob, frame, i ) + unmatch_birth( ob, frame, i ) ) / T;

   VideoObject *e;
   int m_match = 0;
   for( int j=0; j<current.getSize(); j++ ) {
        e = (VideoObject *) current.getEvent(j);
        if( e->child != NULL || e->parent != NULL ) m_match++;
   }

   double accept = 
      energy + log( prior->getWidth() ) + log( prior->getHeight() );
   accept -= log( current.getSize() + 1.0 - m_match );

   if( log(whrg.uniform()) < accept ) { 
     add( ob, frame, i ); 
     Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
   }
   else { delete ob; }

   return;
}

void 
State::death_unmatched( double T )
{
   int frame = whrg.discrete_uniform(0, nframe-1);
   Pattern current = fs[frame].pat;

   VideoObject *e;
   int m_match = 0;
   int j;
   for( j=0; j<current.getSize(); j++ ) {
        e = (VideoObject *) current.getEvent(j);
        if( e->child != NULL || e->parent != NULL ) m_match++;
   }
   if( current.getSize() == m_match ) {
      return;
   }

   int i = whrg.discrete_uniform(1, current.getSize() - m_match);
   int m = 0;
   for( j=0; j<current.getSize(); j++ ) {
        e = (VideoObject *) current.getEvent(j);
        if( e->child == NULL && e->parent == NULL ) m++;
        if( m == i ) { i = j; break; }
   }

   double energy = ( llr_death( frame, i ) + unmatch_death( frame, i ) ) / T;
   double accept = energy + log( current.getSize() - m_match );
   accept -= ( log( prior->getWidth() ) + log( prior->getHeight() ) );

   if( log( whrg.uniform() ) < accept ) { 
      remove( frame, i );
      Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
      delete e; 
   }

   return;
}

void 
State::modify_match( double T)
{
   if( nframe < 2 ) { std::cerr << "Modify match not appropriate\n"; exit(1); }
   int frame1 = whrg.discrete_uniform(0, nframe-2);
   Pattern previous = fs[frame1].pat;

   int j;
   VideoObject *e;
   int nlinked = 0; // the number of linked points 
   for( j=0; j< previous.getSize(); j++ ) {
      e = (VideoObject *) previous.getEvent(j);
      if( e->child != NULL ) nlinked++;
   }

   if( whrg.uniform() < 0.5 ) { // add link 
      if( previous.getSize() - nlinked < 1 ) return;
      if( (fs[frame1+1].pat).getSize() - nlinked < 1 ) return;

      VideoObject *o1;
      int k = whrg.discrete_uniform( 1, previous.getSize() - nlinked );
      int m = 0;
      for( j=0; j<previous.getSize(); j++ ) {
         e = (VideoObject *) previous.getEvent(j);
         if( e->child == NULL ) m++;
         if( m == k ) { o1 = e; break; }
      }
      
      VideoObject *o2;
      k = whrg.discrete_uniform( 1, (fs[frame1+1].pat).getSize() - nlinked );
      m = 0;
      for( j=0; j<(fs[frame1+1].pat).getSize(); j++ ) {
         e = (VideoObject *) (fs[frame1+1].pat).getEvent(j);
         if( e->parent == NULL ) m++;
         if( m == k ) { o2 = e; break; }
      }
      
      double energy = birth_link( o2, o1 ) / T ;
      double accept = energy - log( nlinked + 1.0 );
      accept -= 
         ( log( previous.getSize() - nlinked ) + 
           log( (fs[frame1+1].pat).getSize() - nlinked ) );

      if( log(whrg.uniform()) < accept ) { 
         add_link( o1, o2 ); 
         Hamiltonian -= ( energy * T ); 
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
      }
   }

   else { // remove link
      if( nlinked == 0 ) return;

      int k = whrg.discrete_uniform( 1, nlinked );
      VideoObject *o1;
      VideoObject *o2;
      int m = 0;
      for( j=0; j<previous.getSize(); j++ ) {
         e = (VideoObject *) previous.getEvent(j);
         if( e->child != NULL ) m++;
         if( m == k ) { o1 = e; o2 = (VideoObject *) e->child; break; }
      }

      double energy = ( death_link( o2, o1 ) / T );
      double accept = energy + log( nlinked );
      accept -= 
         ( log( previous.getSize() - nlinked + 1.0 ) +
           log( (fs[frame1+1].pat).getSize() - nlinked + 1.0 ) );

      if( log(whrg.uniform()) < accept ) { 
         remove_link( o1, o2 ); 
         Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
      }
   }

   return;
}

void 
State::modify_index( double T)
{
   int frame = whrg.discrete_uniform(0, nframe-1);
   if( (fs[frame].pat).getSize() <= 1 ) return;
   
   int k = whrg.discrete_uniform(0, (fs[frame].pat).getSize() - 1 );
   VideoObject *ob1 = (VideoObject *) (fs[frame].pat).getEvent( k );

   double hastings = llr_death( frame, k ) / T;
   this->remove( frame, k );

   int l = whrg.discrete_uniform(0, (fs[frame].pat).getSize() - 1 );
   VideoObject *ob2 = (VideoObject *) (fs[frame].pat).getEvent( l );

   hastings += ( llr_death( frame, l ) / T );
   this->remove( frame, l );

   hastings += ( llr_birth( ob1, frame, l ) / T );
   this->add( ob1, frame, l );

   hastings += ( llr_birth( ob2, frame, k ) / T );
   this->add( ob2, frame, k );
   
   if( log(whrg.uniform() ) < hastings ) { 
      Hamiltonian -= ( hastings * T ); 
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
   }
   else { // go back to original permutation
      this->remove( frame, k ); this->remove( frame, l );
      this->add( ob2, frame, l ); this->add( ob1, frame, k );
   }
   
   return;
}

void 
State::modify_object( double T )
{
   int frame = whrg.discrete_uniform(0, nframe-1);
   if( (fs[frame].pat).getSize() < 1 ) return;
   
   int k = whrg.discrete_uniform(0, (fs[frame].pat).getSize() - 1 );
   VideoObject *ob = (VideoObject *) (fs[frame].pat).getEvent( k );
   VideoObject *tob = (VideoObject *) prior->newEvent();
   double accept = log( prior->newLocal( ob, tob, 0.05 ) );
   double energy = ( change(ob, tob, k, frame ) / T );
   accept += energy;

   if( log( whrg.uniform() ) < accept ) { 
     this->remove( frame, k ); 
     this->add( tob, frame, k );
     if( tob->parent != NULL ) (tob->parent)->child = tob;
     if( tob->child != NULL ) (tob->child)->parent = tob;
     Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
     delete ob;
   }
   else { 
      delete tob;
   }
   
   return;
}

void 
State::split_object( double T )
{
   int frame = whrg.discrete_uniform(0, nframe-1);
   Pattern current = fs[frame].pat;
   if( current.getSize() < 1 ) return;

   int k = whrg.discrete_uniform(0, current.getSize() - 1 );
   VideoObject *ob = (VideoObject *) current.getEvent(k);

   VideoObject *s1 = (VideoObject *) prior->newEvent();
   VideoObject *s2 = (VideoObject *) prior->newEvent();
   if( ! prior->split( ob, s1, s2) ) return;

   int oblinks;
   if( whrg.uniform() < 0.5 ) {
      if( ob->parent != NULL ) { s1->parent = ob->parent; }
      if( ob->child != NULL ) { s1->child = ob->child; }
      oblinks = 1;
   }
   else {
      if( ob->parent != NULL ) { s2->parent = ob->parent; }
      if( ob->child != NULL ) { s2->child = ob->child; }
      oblinks = 2;
   }

   int m_singly_up = 0; 
   int m_singly_down = 0; 
   int m_doubly = 0;

   int j;
   VideoObject *e;

   for( j=0; j<current.getSize(); j++ ) {
        e = (VideoObject *) current.getEvent(j);
        if( e->child != NULL && e->parent == NULL ) m_singly_up++;
        if( e->child == NULL && e->parent != NULL ) m_singly_down++;
        if( e->child != NULL && e->parent != NULL ) m_doubly++;
   }

   if( frame < nframe - 1 ) {
      int pos_next = whrg.discrete_uniform(0, 
           (fs[frame+1].pat).getSize() - m_singly_up - m_doubly );
      if( pos_next > 0 ) {
         int m = 0;
         VideoObject *e;
         for( j=0; j<(fs[frame+1].pat).getSize(); j++ ) {
            e = (VideoObject *) (fs[frame+1].pat).getEvent(j);
            if( e->parent == NULL ) m++;
            if( m == pos_next ) { 
               if( oblinks == 1 ) { s2->child = e; }
               if( oblinks == 2 ) { s1->child = e; }
               break;
            }
         }
      }
      // else no link to next frame 
    }
    if( frame > 0 ) {
      int pos_prev = whrg.discrete_uniform(0, 
           (fs[frame-1].pat).getSize() - m_singly_down - m_doubly );
      if( pos_prev > 0 ) {
         int m = 0;
         VideoObject *e;
         for( j=0; j<(fs[frame-1].pat).getSize(); j++ ) {
            e = (VideoObject *) (fs[frame-1].pat).getEvent(j);
            if( e->child == NULL ) m++;
            if( m == pos_prev ) { 
               if( oblinks == 1 ) { s2->parent = e; }
               if( oblinks == 2 ) { s1->parent = e; }
               break;
            }
         }
      }
      // else no link to previous frame 
   }

   double energy = split(ob, s1, s2, oblinks, k, frame ) / T;
   double accept = energy + prior->logSplitrate( ob, s1, s2 );
   accept -= log( 1.0 + current.getSize() );
   if( frame > 0 ) accept += 
      log( 1.0 + (fs[frame-1].pat).getSize() - m_singly_down - m_doubly );
   if( frame < nframe - 1 ) accept +=
      log( 1.0 + (fs[frame+1].pat).getSize() - m_singly_up - m_doubly );

   if( log( whrg.uniform() ) < accept ) {
std::cout << "Split accepted \n"; iwait();
      this->remove( frame, k );
      this->add( s1, frame, k ); 
      this->add( s2, frame, k+1 ); 
      if( s1->child != NULL ) (s1->child)->parent = s1;
      if( s2->child != NULL ) (s2->child)->parent = s2;
      if( s1->parent != NULL ) (s1->parent)->child = s1;
      if( s2->parent != NULL ) (s2->parent)->child = s2;
      Hamiltonian -= ( energy * T );
std::cout << "split Hamiltonian = " << Hamiltonian << "\n";
std::cout << "by potential it is " << potential();
iwait();
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
      delete ob;
   }
   else{ // go back to original
      delete s1; delete s2;
   }

   return;
}

void 
State::merge_object( double T )
{
   int frame = whrg.discrete_uniform(0, nframe-1);
   Pattern current = fs[frame].pat;
   if( current.getSize() < 2 ) return; 
   
   int k = whrg.discrete_uniform(0, current.getSize() - 2 );
   VideoObject *ob1 = (VideoObject *) current.getEvent( k );
   VideoObject *ob2 = (VideoObject *) current.getEvent( k+1 );

   VideoObject *mob = (VideoObject *) prior->newEvent();
   if( ! prior->merge(ob1, ob2, mob) ) return; 

   int oblinks;
   if( whrg.uniform() < 0.5 ) {
      if( ob1->parent != NULL ) { mob->parent = ob1->parent; }
      if( ob1->child != NULL ) { mob->child = ob1->child; }
      oblinks = 1;
   }
   else {
      if( ob2->parent != NULL ) { mob->parent = ob2->parent; }
      if( ob2->child != NULL ) { mob->child = ob2->child; }
      oblinks = 2;
   }

   int m_singly_up = 0; 
   int m_singly_down = 0; 
   int m_doubly = 0;

   VideoObject *e;
   for( int j=0; j<current.getSize(); j++ ) {
      e = (VideoObject *) current.getEvent(j);
      if( e->child != NULL && e->parent == NULL ) m_singly_up++;
      if( e->child == NULL && e->parent != NULL ) m_singly_down++;
      if( e->child != NULL && e->parent != NULL ) m_doubly++;
   }
   if( oblinks == 1 ) {
      if( ob2->child != NULL && ob2->parent == NULL ) m_singly_up--;
      if( ob2->child == NULL && ob2->parent != NULL ) m_singly_down--;
      if( ob2->child != NULL && ob2->parent != NULL ) m_doubly--;
   }
   if( oblinks == 2 ) {
      if( ob1->child != NULL && ob1->parent == NULL ) m_singly_up--;
      if( ob1->child == NULL && ob1->parent != NULL ) m_singly_down--;
      if( ob1->child != NULL && ob1->parent != NULL ) m_doubly--;

   }

   double energy = merge( ob1, ob2, mob, oblinks, k, frame ) / T;
   double accept = energy + prior->logMergerate( ob1, ob2, mob );
   accept += log( current.getSize() );

   if( frame < nframe - 1 ) { accept -= 
       log( 1.0 + (fs[frame+1].pat).getSize() - m_singly_up - m_doubly );
   }
   if( frame > 0 ) { accept -= 
      log( 1.0 + (fs[frame-1].pat).getSize() - m_singly_down - m_doubly );
   }

   if( log( whrg.uniform() ) < accept ) {
      this->remove( frame, k );
      this->remove( frame, k );
      this->add( mob, frame, k );
      if( mob->child != NULL ) (mob->child)->parent = mob;
      if( mob->parent != NULL ) (mob->parent)->child = mob;
      if( oblinks == 1 ) {
         if( ob2->parent != NULL ) ( ob2->parent)->child = NULL;
         if( ob2->child != NULL ) ( ob2->child)->parent = NULL;
      }
      if( oblinks == 2 ) {
         if( ob1->parent != NULL ) ( ob1->parent)->child = NULL;
         if( ob1->child != NULL ) ( ob1->child)->parent = NULL;
      }
      Hamiltonian -= ( energy * T );
if( fabs(Hamiltonian - potential() ) > 1 ) iwait();
      delete ob1; delete ob2;
   }
   else{ // go back to original
      delete mob;
   }


   return;
}


double
State::potential()
{
   double u = 0;
   double v = 0;
   double w = 0;

   int i;
   VideoObject *e;

   for( int f=0; f<nframe; f++ ) {
      FOR( p over (fs[f].data) ) 
        u += noise->logCanonic( (fs[f].data)(p), (fs[f].sil)->val(p) ); 

std::cout << "L2 part " << u << "\n";

      Pattern current;
      for( i=0; i<(fs[f].pat).getSize(); i++ ) {
         e = (VideoObject *) (fs[f].pat).getEvent(i);
         v += ( prior->logCond_intens( current, e, i) +
                log( 1.0 + current.getSize() ) );
         current.add( e );

         if( f < nframe - 1 ) {
           if( e->child != NULL ) w += prior->logRho( e, e->child );
           else w += prior->logLambda(e);
         }
         if( f > 0 ) {
           if(  e->parent == NULL ) w += prior->logLambda(e);
         }
      }

std::cout << "prior part " << v << "\n";
std::cout << "links part " << w << "\n";

   }

   return( - ( u + v + w ) );
}

State::State( FrameState *cfs, int cf, Noise *cn, SquareStrauss *cprior )
{
   nframe = cf; fs = cfs;
   noise = cn; prior = cprior;

   Hamiltonian = potential();

   std::cout << "Hamiltonian = " << Hamiltonian << "\n";
}

void State::report_all() const
{
   VideoObject *e;
   for( int f=0; f<nframe; f++ ) {
      std::cout << "frame " << f << ": \n";
      Pattern current = fs[f].pat;
      for( int i=0; i<current.getSize(); i++ ) {
         e = (VideoObject *) current.getEvent(i);
         e->report();
      }
      std::cout << "\n";
   }
   return;     
}

void State::iter( int nsteps, double T )
{
   int w;
   for( int i=0; i<nsteps; i++ ) {
 std::cerr << "Iter: i = " << i << "; ";

   w = whrg.discrete_uniform(1, 12);
   switch(w) {
      case 1: { birth_matched(T); break; }
      case 2: { death_matched(T); break; }
      case 3: { birth_unmatched(T); break; }
      case 4: { death_unmatched(T); break; }
      case 5: { modify_index(T); break; }
      case 6: { modify_match(T); break; }
      case 7: { modify_object(T); break; }
      case 8: { modify_object(T); break; }
      case 9: { modify_object(T); break; }
      case 10: { modify_object(T); break; }
      case 11: { split_object(T); break; }
      case 12: { merge_object(T); break; }
      default: { std::cerr << "Switch: wrong case\n"; exit(1); }
      }

/*
if(this->report_Hamiltonian() < 195 ) {
std::cout << "w= " << w << "\n";
std::cout << "Hamiltonian = " << this->report_Hamiltonian() << "\n";
std::cout << "potential = " << potential() << "\n";
this->report_all();
exit(1);
}

if( this->report_Hamiltonian() > potential() + 5 || this->report_Hamiltonian() < potential() - 5) {
std::cout << "w= " << w << "\n";
std::cout << "Hamiltonian = " << this->report_Hamiltonian() << "\n";
std::cout << "potential = " << potential() << "\n";
this->report_all();
exit(1);
}
*/

}
   return;
}

