#ifndef _state_
#define _state 1

#include "signal.h"
#include "noise.h"
#include "prior.h"
#include "strauss.h"

class FrameState {
 public:
   ipShort data;
   Pattern pat;
   Signal *sil;
};

class State {
   int nframe;
   FrameState *fs;

   const Noise *noise;
   SquareStrauss *prior;

   float Hamiltonian;
   int has( point p ) { return ( (fs[0].data).has(p) ); }

/* 
  contributions to Hastings ratio by prior and likelihood
  ON THE LOG SCALE !
*/
   double llr_birth( VideoObject *ob, int frame, int i );
   double llr_death( int frame, int i );

   double down_birth( const VideoObject *ob, int frame, int i, const VideoObject *prev );
   double up_birth( const VideoObject *ob, int frame, int i, const VideoObject *next );
   double double_birth( const VideoObject *ob, int frame, int i, const VideoObject *prev, 
                        const VideoObject *next );
   double unmatch_birth( const VideoObject *ob, int frame, int i );

   double down_death( int frame, int i );
   double up_death( int frame, int i );
   double double_death( int frame, int i );
   double unmatch_death( int frame, int i );

   double birth_link( const VideoObject *ob, const VideoObject *prev ) const; 
   double death_link( const VideoObject *ob, const VideoObject *prev ) const; 

   double change( VideoObject *ob, VideoObject *tob, int i, int frame );

   double split( VideoObject *ob, VideoObject *s1, VideoObject *s2, 
                 int link, int i, int frame );
   double merge( VideoObject *ob1, VideoObject *ob2, VideoObject *mob , 
                 int link, int i, int frame );

/*
   MH updates
*/

   void birth_matched(double T);
   void birth_singly_up( int frame, int m_s, int m_d, double T );
   void birth_singly_down( int frame, int m_s, int m_d, double T );
   void birth_doubly( int frame, int m_su, int m_sd, int m_d, double T );
   void birth_unmatched(double T);
   void death_matched(double T);
   void death_singly_up( int frame, int m_s, int m_d, double T );
   void death_singly_down( int frame, int m_s, int m_d, double T );
   void death_doubly( int frame, int m_su, int m_sd, int m_d, double T );
   void death_unmatched(double T);
   void modify_match(double T);
   void modify_index(double T);
   void modify_object(double T);
   void split_object(double T);
   void merge_object(double T);

public:
   State( FrameState *cfs, int cf, Noise *cn, SquareStrauss *cprior );
   void iter( int nsteps, double T);
      // number of steps and temperature (for annealing purposes)
   FrameState *getState() const { return fs; }
   int getFrames() const { return nframe; }
   void report() const {
   	std::cerr << "Blur free state class for posterior sampling\n"; }
   void report_all() const;
   double report_Hamiltonian() const { return Hamiltonian; } 
   double potential();
/* DEBUG ONLY */
void set_Hamiltonian( double h) { Hamiltonian = (float) h; }

   void add( VideoObject *ob, int frame, int i ); // no links
   void add_link( VideoObject *o1, VideoObject *o2 );
   void remove( int frame, int i ); // not its links
   void remove_link( VideoObject *o1, VideoObject *o2 );
};

   
#endif
