#ifndef _sc_
#define _sc_ 1

#include "prior.h"
#include "event.h"

class SquareStrauss : public Prior {
   float logbeta; 
   float loggamma; 
   int ngrey; 
   int bgcolour; // background
   float maxside;
   float minside; // upper bounds on colour and side length of squares
   float var; // the variance of the Gaussian displacement kernel
 public:
   SquareStrauss( double clb, double clg, double cw, double ch, int cng, int cbg,
                  double cmax, double cmin, double var );
   double cond_intens( const Pattern& p, const Event *e, int i ) const 
     { return ( exp( logCond_intens(p, e, i) ) ); }
   double logCond_intens( const Pattern& p, const Event *e, int i ) const;
   int monotone() const;

   double getLogGamma() const { return loggamma; }
   double getLogBeta() const { return logbeta; }
   double maxSide() const { return maxside; }
   int backColour() const { return bgcolour; }
   void setParameters( double cb, double cg );
   int getInteractingPairs( const Pattern& p ) const;
      // sum_{i<j} 1{  R(x_i) \cap R(x_j) \neq \emptyset }

   void report() const;
   void printParameters();
      // by default the ubnd
   void printSample( const Pattern& p );
   void printStatistics( const Pattern& p );
   Event *newEvent() const;
   double newLocal( Event *, Event *, double ) const;

   double logLambda( const Object *) const { return( -10.0 ); }
     // constant or as function of distance to boundary
   double logRho( const Object *, const Object *) const;
   Object *from_kernel( const Object * ) const;
   double logKernel( const Object *current, const Object *next ) const;

   // split and merge

   double logSplitrate( const Object *, const Object *, const Object * ) const;
   double logMergerate( const Object *, const Object *, const Object * ) const;
   bool split( const Object *, Object *, Object * ) const;
   bool merge( const Object *, const Object *, Object * ) const;
};

#endif
